(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA (Commissariat  l'nergie atomique et aux nergies              *)
(*         alternatives)                                                  *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version 2.1                 *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

open Cil_types
open Format

(* [JS 2010/05/21] Implementation of the internal state could be better:
   just apply one functor, not two
   (at least allow to directly export [self]). *)

(* [JS 2010/09/07] MessageMap is an hashtbl... Hum! *)
module MessageMap =
  State_builder.Hashtbl
    (Cil_datatype.Int_hashtbl)
    (Datatype.Make
       (struct
	 include Datatype.Serializable_undefined
	 open Log
	 type t = event
	 let name = "message"
	 let reprs =
	   [ { evt_kind = Failure;
	       evt_plugin = "";
	       evt_source = None;
	       evt_message = "" } ]
	 let mem_project = Datatype.never_any_project
	end))
    (struct
       let name = "message_table"
       let size = 17
       let dependencies = []
       let kind = `Internal
     end)

module MessageCounter =
  State_builder.Zero_ref
    (struct
       let name = "message_counter"
       let dependencies = []
       let kind = `Internal
     end)

let depend from =
  State_dependency_graph.Dynamic.add_dependencies
    ~from
    [ MessageMap.self; MessageCounter.self ]

(* [JS 2010/09/27] not used anymore.
   If useful again, should also clear its dependencies accordingly *)
(*let clear () = MessageCounter.clear () ; MessageMap.clear ()*)

let iter f = MessageMap.iter f

let enable_collect =
  let enabled = ref false in
  fun () ->
    if !enabled = false then (
      enabled := true;
      Cilmsg.debug "Enable collection of error messages." ;
      let emit e =
        let c = MessageCounter.get () in
        MessageMap.add c e ;
        MessageCounter.set (succ c)
      in
      begin
        Log.add_listener ~kind:[Log.Error;Log.Warning] emit ;
      end)

let disable_echo () =
  begin
    Cilmsg.debug "Disable echo for error messages" ;
    Log.set_echo ~kind:[Log.Error;Log.Warning] false ;
  end

let dump_messages () = MessageMap.iter (fun _ e -> Log.echo e)

(*
Local Variables:
compile-command: "make -C ../.."
End:
*)
