(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA   (Commissariat  l'nergie atomique et aux nergies            *)
(*           alternatives)                                                *)
(*    INRIA (Institut National de Recherche en Informatique et en         *)
(*           Automatique)                                                 *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version v2.1                *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

(** ACSL comparable property.
    @since Carbon-20101201 *)

open Cil_types
open Db_types

type identified_complete = kernel_function * kinstr * string list
type identified_disjoint =  identified_complete
type identified_code_annotation = kernel_function * stmt * code_annotation
type identified_assigns =
    kernel_function
    * kinstr
    * funbehavior option (* loop assigns are not part of a given behavior. *)
    * identified_term assigns list
type identified_decrease = kernel_function * kinstr * term variant
type identified_behavior = kernel_function * kinstr * funbehavior

type predicate_kind =
  | PKRequires of funbehavior
  | PKAssumes of funbehavior
  | PKEnsures of funbehavior * termination_kind
  | PKTerminates

type identified_predicate =
    predicate_kind * kernel_function * kinstr * Cil_types.identified_predicate

type identified_spec = kernel_function * kinstr * funspec

type identified_property =
  | IPBlob of State.t (* an unidentified property *)
  | IPPredicate of identified_predicate
  | IPAxiom of string
  | IPCodeAnnot of identified_code_annotation
  | IPComplete of identified_complete
  | IPDisjoint of identified_disjoint
  | IPAssigns of identified_assigns
  | IPDecrease of identified_decrease
  | IPBehavior of identified_behavior

include Datatype.S_with_collections with type t = identified_property

val get_kinstr: identified_property -> kinstr
val get_kf: identified_property -> kernel_function option
val get_behavior: identified_property -> funbehavior option

(*
Local Variables:
compile-command: "make -C ../.."
End:
*)
