(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA   (Commissariat  l'nergie atomique et aux nergies            *)
(*           alternatives)                                                *)
(*    INRIA (Institut National de Recherche en Informatique et en         *)
(*           Automatique)                                                 *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version v2.1                *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

open Cil_datatype

type fvar

type data = private
    D1null
  | D1base of data
  | D1addr of Cil_types.varinfo * Formula.tau
  | D1var of (Cil_types.varinfo * Clabels.c_label) option * fvar
  | D1depl of data * epath
  | D1shift of data * dterm
  | D1proj of data * epath
  | D1indir of data
  | D1mu of data * epath * dterm option
  | D1muRange of data * dterm (* mu (tab, [.(i..j)]) *)
  | D1muI of data * dterm option * dterm option
  | D1muIrange of data * dterm (* mu (p, [#(i...j), * ]) *)
and epath =
  | Pfield of Cil_types.fieldinfo
  | Pidx of dterm
and dterm = data Fol.d_term

val eq_data : data -> data -> bool
val eq_var : fvar -> fvar -> bool

val pp_data : Format.formatter -> data -> unit
val pp_var : Format.formatter -> fvar -> unit

include Formula.S with type pool = unit
                  and type var = fvar
                  and type 'a term = dterm
                  and type pred = dterm Fol.t_pred

val ctype_of_fvar : fvar -> Cil_types.typ option

val data_of_cvar_at_l :
  fvar Varinfo.Hashtbl.t -> Clabels.c_label -> Cil_types.varinfo ->
  (Cil_types.varinfo -> Formula.tau) -> data

val fvar_of_cvar_at : create:bool ->
  (fvar Varinfo.Hashtbl.t) -> Clabels.c_label -> Cil_types.varinfo ->
  (Cil_types.varinfo -> Formula.tau) -> fvar

val addr : Cil_types.varinfo -> Formula.tau -> data
val null : data
val base : data -> data
val shift_field : data -> Cil_types.fieldinfo -> data
val shift_index : data -> dterm -> data
val shift_epath : data -> epath -> data
val shift_pointer : data -> dterm -> data
val proj : data -> epath -> dterm
val indir : (fvar Varinfo.Hashtbl.t) -> Clabels.c_label -> data -> dterm
val mk_mu : data -> epath -> dterm option -> data
val mk_mu_range : data -> dterm -> data
val mk_mu_sh_indir : data -> dterm option -> dterm option -> data 
val mk_mu_range_indir : data -> dterm -> data

val addr_of_data : data -> data

val term_of_var : fvar -> dterm
val term_of_data : data -> dterm
val data_of_term : dterm -> data

val visit_data_in_pred : all:bool ->
  (('a -> dterm -> 'a) -> 'a -> data -> 'a) -> 'a -> pred -> 'a
val subst_data_in_pred : (data -> data) -> pred -> pred

