(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA   (Commissariat  l'nergie atomique et aux nergies            *)
(*           alternatives)                                                *)
(*    INRIA (Institut National de Recherche en Informatique et en         *)
(*           Automatique)                                                 *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version v2.1                *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

open Cil_types
open Ctypes
open Formula

(**
   [identifier s] strip all non alpha-numerical characters in [s].
*)

val identifier : string -> string

(** Returns [true] iff the string contains '_' and the prefix ending at the first '_' is
    a reserved prefix.
    In this case the string cannot be used as an identifier in the generated files.
*)
val has_reserved_prefix: string -> bool

val register_prefix : string -> unit

module Tau :
sig
  val tau_of_logic_ctype : Cil_types.typ -> tau
  val pp_tau : Format.formatter -> tau -> unit
end

module Cvalues(M : Mvalues.Model) : Mvalues.Values
  with type loc = M.loc
  and module D = M.D
  and module A = M.A
  and module R = M.R

module Create (F:Formula.S) :
sig

  include Mdata.S with module F = F

end

module Collector : sig
  type t

  val push : unit -> t
  val pop :
      t -> Wpo.warning list * Wpo.dependency list

  (**
   * When adding a warning, one has to provide :
   * - the source of the warning (for instance "model M"),
   * - the effect of the warning (for instance "stop computation")
   * - and a formated message about why this warning is emited.
  *)
  val add_warning : ?severe:bool -> ?source:string -> reason:string ->
    ('a, Format.formatter, unit) format -> 'a

  val add_depend :
    WpAnnot.prop_id -> ('a, Format.formatter, unit) format -> 'a

end

(*
Local Variables:
compile-command: "make -C ../.."
End:
*)
