(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA   (Commissariat  l'nergie atomique et aux nergies            *)
(*           alternatives)                                                *)
(*    INRIA (Institut National de Recherche en Informatique et en         *)
(*           Automatique)                                                 *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version v2.1                *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

open Ctypes
open Formula

(* ------------------------------------------------------------------------ *)
(* ---  Logic Terms Constructors                                        --- *)
(* ------------------------------------------------------------------------ *)

(** Fol specialized with data type = (unit) *)

type data = unit
type uterm = data Fol.d_term
type predicate = uterm Fol.t_pred
type decl = predicate Formula.declaration

module Vernacular =
struct

  type 'a term = uterm
  type pred = predicate

  include Fol_gformula.Langage

end

include Vernacular
include Whylib.Create(Vernacular)
include Fol_gformula.Pool
include Fol_gformula.Predicate

let term_has_var = Fol.e_has_var (fun _ () -> false)
let pred_has_var = Fol.p_has_var term_has_var

let term_closed t = Fol.bad_term_closed [] t
let pred_closed p = Fol.bad_pred_closed [] p

type alpha = Fol.Var.t Fol.Vmap.t
let empty_alpha = Fol.Vmap.empty
let fold_alpha = Fol.Vmap.fold
let p_more_alpha_cv = Fol.pred_alpha_cv (fun _ () -> assert false)
let p_alpha_cv p = Fol.p_alpha_cv (fun _ () -> assert false) p

module PrettyUnit = Fol_pretty.Create
  (struct
     type d = unit
     let pp_data _fmt () = ()
     let rec pp_term fmt t = Fol_pretty.fpretty_term pp_term pp_data fmt t
   end)

include PrettyUnit
let pp_var fmt x = pp_term fmt (Fol.e_var x)

let huge_term = Fol.huge_term
let huge_pred = Fol.huge_pred
