(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA   (Commissariat  l'nergie atomique et aux nergies            *)
(*           alternatives)                                                *)
(*    INRIA (Institut National de Recherche en Informatique et en         *)
(*           Automatique)                                                 *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version v2.1                *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

(* ------------------------------------------------------------------------ *)
(** Splitting Goals                                                         *)
(* ------------------------------------------------------------------------ *)

(** {2 Zone-unions Flattening} 
    The [unfolding] methods below only applies to variables that holds
    a union of [zone] values. Other variables are left letified. *)

module Env : Map.S with type key = Fol.Var.t

val e_unfold : unit Fol.d_term Env.t -> unit Fol.d_term -> unit Fol.d_term
val p_unfold : unit Fol.d_term Env.t -> unit Fol.d_pred -> unit Fol.d_pred

(** {2 Generic Case Split} *)

val dispatch : 'd Fol.d_pred -> 'd Fol.d_pred Bag.t
  (** Splits conjunctions and unfold the definition
      of [included] over [zones]. *)

(** {2 Splitter Interface} *)

type pred = unit Fol.d_pred
val split : Mcfg.assigns_method -> pred -> pred Bag.t
  (** First, unfold zones when method is effect-assigns.
      Then applies [dispatch]. *)
