(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA   (Commissariat  l'nergie atomique et aux nergies            *)
(*           alternatives)                                                *)
(*    INRIA (Institut National de Recherche en Informatique et en         *)
(*           Automatique)                                                 *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version v2.1                *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

(* -------------------------------------------------------------------------- *)
(* --- WHY Export                                                         --- *)
(* -------------------------------------------------------------------------- *)

open Formula
open Format

type pred = unit Fol.d_pred
    
let rec export_tau fmt = function
  | Integer -> pp_print_string fmt "int"
  | Real -> pp_print_string fmt "real"
  | Boolean -> pp_print_string fmt "bool"
  | Pointer -> pp_print_string fmt "pointer"
  | Record _ -> pp_print_string fmt "data farray"
  | Array arr ->
      Format.fprintf fmt "%a farray"
	export_tau (Datalib.Tau.tau_of_logic_ctype arr.Ctypes.arr_element)
  | Set te ->
      Format.fprintf fmt "%a set" export_tau te
  | ADT(s,[]) -> pp_print_string fmt s
  | ADT(s,[t]) -> Format.fprintf fmt "%a %s" export_tau t s
  | ADT(s,t::ts) ->
      Format.fprintf fmt "@[(%a" export_tau t ;
      List.iter (fun t -> Format.fprintf fmt ",@,%a" export_tau t) ts ;
      Format.fprintf fmt ") %s@]" s
	
let export_data _ _ = ()

let rec export_term fmt t = Fol_pretty.fpp_term export_term export_data fmt t
  
let rec pp_pred_atom fmt p =
  Fol_pretty.epp_pred_atom {
    Fol_pretty.pp_type = export_tau ;
    Fol_pretty.pp_term = export_term ;
    Fol_pretty.pp_pred = pp_pred_atom ;
  } fmt p
    
let export_pred fmt p =
  Fol_pretty.epp_pred_vbox {
    Fol_pretty.pp_type = export_tau ;
    Fol_pretty.pp_term = export_term ;
    Fol_pretty.pp_pred = pp_pred_atom ;
  } fmt p
    
let export_item fmt name item = Fol_pretty.fpp_item export_pred export_tau fmt name item
  
let export_decl fmt d =
  Pretty_utils.pp_trail Fol_pretty.fpp_header fmt d ;
  Format.pp_print_newline fmt () ;
  Fol_pretty.fpp_item export_pred export_tau fmt d.d_name d.d_item
    
let export_goal fmt x g = Fol_pretty.fpp_goal export_pred fmt x g
