(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA   (Commissariat  l'nergie atomique et aux nergies            *)
(*           alternatives)                                                *)
(*    INRIA (Institut National de Recherche en Informatique et en         *)
(*           Automatique)                                                 *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version v2.1                *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

(* -------------------------------------------------------------------------- *)
(* --- Basic Implementation (Arithmetics is mapped to Z)                  --- *)
(* -------------------------------------------------------------------------- *)

open Ctypes
open Formula

module Create (D:Mdata.S) : (Mint.S with module D = D) =
struct

  module D = D
    
  (* integer cst *)
  let  i_int _i k = D.F.e_icst k

  (* arithmetic operations on Z in why *)
    
  let i_neg i e = 
    let k = D.F.e_ineg e in 
    if (Ctypes.signed i) then  k else D.modulo i k 
  let i_op _i iop e1 e2 = D.F.e_iop iop e1 e2
  let i_cmp _i cmp e1 e2 = (D.F.e_icmp cmp e1 e2)
    
  (* bitwise operations on Z in why *)

  let sizeof i = D.F.e_int (Ctypes.i_sizeof i)
  let signess i = if (Ctypes.signed i) then D.F.e_true else D.F.e_false

  let bits_not  _ = D.e_app1 "int_not" 
  let bits_and  _ = D.e_app2 "int_and" 
  let bits_or   _ = D.e_app2 "int_or" 
  let bits_xor  _ = D.e_app2 "int_xor" 
  let bits_lshift  _ = D.e_app2 "int_lsh" 
  let bits_rshift  i = 
    D.e_app2 (if (Ctypes.signed i) then "int_rshs" else "int_rshu")

  (* integers conversions *)  

  let i_convert t1 t2 e =  
    if Ctypes.sub_c_int t1 t2 then e else D.modulo t2 e

  let integer_of_int _ e = e 
    (* Z-representatives live in the correct range. *)

  let int_of_integer _ e = e 
    (* Conversions are never necessary for this model, since no overflow are allowed. *)

end
