(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA   (Commissariat  l'nergie atomique et aux nergies            *)
(*           alternatives)                                                *)
(*    INRIA (Institut National de Recherche en Informatique et en         *)
(*           Automatique)                                                 *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version v2.1                *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

(* -------------------------------------------------------------------------- *)
(** Model for the interpretation of ACSL/C                                    *)
(* -------------------------------------------------------------------------- *)

open Ctypes
open Clabels
open Formula
open Cil_types
open Mdata

module type S =
sig

  (** {2 Term Values} *)

  include Mvalues.S

  (** {2 Frame Environment} *)

  type frame

  val new_frame: Kernel_function.t ->
    m_here:mem option -> 
    m_pre:mem option -> 
    m_post:mem option -> 
    D.F.var option ->
    frame

  val mem_at : frame -> c_label -> mem 
    (** [mem_at e l] returns the memory associates to
	the label [l] in the environment [e]. *)
    
  val result : frame -> D.F.var * Cil_types.typ
    (** [result e] returns the variable associates to 
	the function result in the environment [e].*)
    
  val exit_status : frame -> D.F.var
    (** [exit_status e] returns the variable associates to 
	the function exit status in the environment [e].*)

  val set_exit_status : frame -> D.F.var -> frame
  val get_exit_status : frame -> D.F.var option
     
  (** {2 Pointers} *)
    
  val base_address : mem -> loc -> loc
    (** [base_address m l] return the [base address] of [l].*)
    
  val block_length : mem -> loc -> integer D.F.term
    (** [block_length m l t] return the [block_length] 
	of the location [l]. *)
    
  (** {2 Validity }*)
    
  val valid : mem -> loc D.assigned -> D.F.pred

  (** {2 Separation} *)

  val separated : loc D.assigned -> loc D.assigned -> D.F.pred

  (** {2 User-defined Predicates} *)

  type closure

  val userdef_frame : unit -> frame

  val userdef_signature : 
    frame -> abstract D.F.term list -> D.F.pred list ->
    D.F.var list * closure list

  val label_at_closure : closure -> string

  val userdef_closure : mem -> closure -> abstract D.F.term

  val pp_closure : Format.formatter -> closure -> unit

end
