(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA   (Commissariat  l'nergie atomique et aux nergies            *)
(*           alternatives)                                                *)
(*    INRIA (Institut National de Recherche en Informatique et en         *)
(*           Automatique)                                                 *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version v2.1                *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

(* ------------------------------------------------------------------------ *)
(* ---  Translation of Term and Predicates                              --- *)
(* ------------------------------------------------------------------------ *)

open Ctypes
open Clabels
open Formula
open Cil_types

module Create
  (M : Mlogic.S)

  :
sig

  (** {2 Translation environments} *)

  type env

  val env_body : M.frame -> env
  val env_at : env -> c_label -> env

  val call_post : 
    Kernel_function.t -> M.value list -> M.mem -> M.mem -> 
    M.D.F.var option ->  env
  
  val call_pre  : Kernel_function.t -> M.value list -> M.mem -> env

  val call_exit : env -> M.D.F.var ->  env

  val assigns_clause : M.frame -> env

  val get_frame : env -> M.frame

  (** {2 Translation functions} *)

  (** [term e t] interprets the C terms [t] in memory model environment [e]
      as a logic term.**)
  val term : env -> term -> abstract M.D.F.term

  (** [prop e p] interprets an ACSL predicate as a logic predicats 
      in memory model environment [e]. **)
  val prop : env -> predicate named -> M.D.F.pred

  (** Compiles an arbitrary term representing a set of left-values into a zone *)
  val assigned : env -> Cil_types.term -> M.loc M.D.assigned list

  (** {2 Axiomatics management} *)

  (** Compile an axiom and add it to the list of global declarations. *) 
  val add_axiom : string -> Cil_types.logic_label list -> Cil_types.predicate named -> unit

  type instance
  val as_hint : instance -> M.D.hint (* identity *)

  val push_instances : string -> instance list -> unit
  val pop_instances : string -> instance list
  val instantiate : instance list -> M.D.F.pred -> instance list * M.D.F.pred
  val merge : instance list -> instance list -> instance list

end
