(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA   (Commissariat  l'nergie atomique et aux nergies            *)
(*           alternatives)                                                *)
(*    INRIA (Institut National de Recherche en Informatique et en         *)
(*           Automatique)                                                 *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version v2.1                *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

open Cil
open Cil_types
open Ctypes

module AddrTaken =
  State_builder.Hashtbl
    (Cil_datatype.Varinfo.Hashtbl)
    (Datatype.Unit)
    (struct let name = "WP: addr_taken"
            let dependencies = [Ast.self]
            let kind = `Internal
            let size = 47
     end)

let set_addr_taken vinfo =
  if Wp_parameters.debug_atleast 1 && not (AddrTaken.mem vinfo)
  then Wp_parameters.debug "Non-functional variable %a"
    !Ast_printer.d_var vinfo ;
  AddrTaken.replace vinfo ()

class address_taken_var = object
  inherit Visitor.frama_c_inplace
  method vexpr e =
    match (Cil.stripInfo e).enode with
      | StartOf (Var vinfo,_) | AddrOf (Var vinfo,_) ->
	  set_addr_taken vinfo ; DoChildren
      | _ ->
	  DoChildren

  method vterm term = match term.term_node with
    | TAddrOf(TVar{lv_origin=Some vinfo},_) 
    | TStartOf(TVar{lv_origin=Some vinfo},_) ->
	set_addr_taken vinfo ; DoChildren
    | _ -> DoChildren
	

end
  
let compute () =
  if not (AddrTaken.is_computed ()) then
    begin
      Wp_parameters.debug "Computing logical variables.";
      Visitor.visitFramacFile (new address_taken_var) (Ast.get()) ;
      AddrTaken.mark_as_computed ()
    end

let is_funvar vinfo =
  Wp_parameters.LogicVar.get() &&
    begin
      if vinfo.vglob && Parameters.LibEntry.get() then false 
      else (compute () ; not (AddrTaken.mem vinfo))
    end



