(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA   (Commissariat  l'nergie atomique et aux nergies            *)
(*           alternatives)                                                *)
(*    INRIA (Institut National de Recherche en Informatique et en         *)
(*           Automatique)                                                 *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version v2.1                *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

(* -------------------------------------------------------------------------- *)
(* --- PRELUDE.why                                                        --- *)
(* -------------------------------------------------------------------------- *)
  
let neg_int = "neg_int"
let add_int = "add_int"
let sub_int = "sub_int"
let mul_int = "mul_int"
let div_int = "computer_div"
let mod_int = "computer_mod"

let eq_int = "eq"
let ne_int = "neq"
let lt_int = "lt_int"
let le_int = "le_int"

(* -------------------------------------------------------------------------- *)
(* --- BOOL.why                                                           --- *)
(* -------------------------------------------------------------------------- *)

let bool_not = "bool_not"
let bool_and = "bool_and"
let bool_or  = "bool_or"

(* -------------------------------------------------------------------------- *)
(* --- INTEGERS.why                                                       --- *)
(* -------------------------------------------------------------------------- *)
  
let eq_int_bool = "eq_int_bool"
let ne_int_bool = "neq_int_bool"
let lt_int_bool = "lt_int_bool"
let le_int_bool = "le_int_bool"

(* -------------------------------------------------------------------------- *)
(* --- REAL.why                                                           --- *)
(* -------------------------------------------------------------------------- *)
  
let neg_real = "neg_real"
let add_real = "add_real"
let sub_real = "sub_real"
let mul_real = "mul_real"
let fract_real = "div_real"
  
let eq_real_bool  = "eq_real_bool"
let ne_real_bool  = "neq_real_bool"
let lt_real_bool  = "lt_real_bool"
let le_real_bool  = "le_real_bool"

let eq_real  = "eq_real"
let ne_real  = "neq_real"
let lt_real  = "lt_real"
let le_real  = "le_real"

let integer_of_real = "truncate_real_to_int"
let real_of_integer = "real_of_int"

(* -------------------------------------------------------------------------- *)
(* --- OPERATORS                                                          --- *)
(* -------------------------------------------------------------------------- *)

open Formula

let i_pred = function
  | Ceq  -> eq_int
  | Cneq -> ne_int
  | Clt  -> lt_int
  | Cleq -> le_int

let i_bool = function
  | Ceq  -> eq_int_bool
  | Cneq -> ne_int_bool
  | Clt  -> lt_int_bool
  | Cleq -> le_int_bool

let i_op = function
  | Iadd -> add_int
  | Isub -> sub_int
  | Imul -> mul_int
  | Idiv -> div_int
  | Imod -> mod_int

let r_pred = function
  | Ceq  -> eq_real
  | Cneq -> ne_real
  | Clt  -> lt_real
  | Cleq -> le_real

let r_bool = function
  | Ceq  -> eq_real_bool
  | Cneq -> ne_real_bool
  | Clt  -> lt_real_bool
  | Cleq -> le_real_bool

let r_op = function
  | Radd -> add_real
  | Rsub -> sub_real
  | Rmul -> mul_real
  | Rdiv -> fract_real

module Create(F : Vernacular) =
struct

  let unop f a = F.unwrap (F.e_call f [F.wrap a])
  let binop f a b = F.unwrap (F.e_call f [F.wrap a;F.wrap b])
  let predop f a b = F.p_call f [F.wrap a;F.wrap b]

  let e_ineg = unop neg_int
  let e_rneg = unop neg_real
    
  let e_icmp op = binop (i_bool op)
  let p_icmp op = predop (i_pred op)
    
  let e_rcmp op = binop (r_bool op)
  let p_rcmp op = predop (r_pred op)
    
  let e_iop op = binop (i_op op)
  let e_rop op = binop (r_op op)

  let real_of_integer = unop real_of_integer
  let integer_of_real = unop integer_of_real

  let a_true = F.wrap (F.e_int 1)
  let a_false = F.wrap (F.e_int 0)

  let e_bool (c : boolean F.term) : integer F.term = 
    F.unwrap (F.e_call "ite" [F.wrap c;a_true;a_false])

  let e_cond (c : boolean F.term) (a : 'a F.term) (b : 'a F.term) : 'a F.term =
    F.unwrap (F.e_call "ite" [F.wrap c;F.wrap a;F.wrap b])

  let e_not = unop bool_not
  let e_and = binop bool_and
  let e_or  = binop bool_or

  let e_bnot   = unop "bnot"
  let e_band   = binop "band"
  let e_bor    = binop "bor"
  let e_bxor   = binop "bxor"
  let e_lshift = binop "lshift"
  let e_rshift = binop "rshift"

    
end
