(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA   (Commissariat  l'nergie atomique et aux nergies            *)
(*           alternatives)                                                *)
(*    INRIA (Institut National de Recherche en Informatique et en         *)
(*           Automatique)                                                 *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version v2.1                *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

open Formula

module Create(F:Vernacular) :
sig

  val e_ineg : integer F.term -> integer F.term
  val e_rneg : real F.term -> real F.term
  val e_iop  : integer_op -> integer F.term -> integer F.term -> integer F.term
  val e_rop  : real_op -> real F.term -> real F.term -> real F.term
  val e_icmp : cmp_op -> integer F.term -> integer F.term -> boolean F.term
  val e_rcmp : cmp_op -> real F.term -> real F.term -> boolean F.term
  val p_icmp : cmp_op -> integer F.term -> integer F.term -> F.pred
  val p_rcmp : cmp_op -> real F.term -> real F.term -> F.pred

  val e_bool : boolean F.term -> integer F.term
  val e_not  : boolean F.term -> boolean F.term
  val e_and  : boolean F.term -> boolean F.term -> boolean F.term
  val e_or   : boolean F.term -> boolean F.term -> boolean F.term
  val e_cond : boolean F.term -> 'a F.term -> 'a F.term -> 'a F.term
    
  val e_bnot   : integer F.term -> integer F.term
  val e_band   : integer F.term -> integer F.term -> integer F.term
  val e_bor    : integer F.term -> integer F.term -> integer F.term
  val e_bxor   : integer F.term -> integer F.term -> integer F.term
  val e_lshift : integer F.term -> integer F.term -> integer F.term
  val e_rshift : integer F.term -> integer F.term -> integer F.term

  val integer_of_real : real F.term -> integer F.term
  val real_of_integer : integer F.term -> real F.term

end

