(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA   (Commissariat  l'nergie atomique et aux nergies            *)
(*           alternatives)                                                *)
(*    INRIA (Institut National de Recherche en Informatique et en         *)
(*           Automatique)                                                 *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version v2.1                *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

open Cil_types

(** Build [ xi = Init (i) /\ ...] forall inputs part of the assigns property. *)
val mk_assign_pre : identified_term assigns -> identified_predicate

(** Build [P(out)] where [out] is the left part of the assigns property. 
Process [\result] and [\exit_status] according to [termination_kind].
**)
val mk_assign_post : termination_kind -> identified_term assigns 
  -> identified_predicate option

(** For each behavior of the specification, return a predicate
* which is a conjonction of [ out_i = implicit_fun_i (inputs)] 
* for each [out_i \from inputs] assigns property of the behavior.
* [implicit_fun_i] is the implicit fonction for the output. 
* [kf] and [ki] give information to know there the specification comes from
* in order to build the names for the implicit functions.
* [termination_kind] is used to filter [\result] and [\exit_status] when needed.
* *)
val post_of_assigns_in_spec : Kernel_function.t -> kinstr -> 
  funspec -> termination_kind -> (funbehavior * identified_predicate) list
