(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA   (Commissariat  l'nergie atomique et aux nergies            *)
(*           alternatives)                                                *)
(*    INRIA (Institut National de Recherche en Informatique et en         *)
(*           Automatique)                                                 *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version v2.1                *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

include Plugin.S

val reset : unit -> unit

(** {2 Goal Selection} *)

module WP: Plugin.BOOL
module Functions: Plugin.STRING_SET
module Behaviors: Plugin.STRING_SET
module Properties: Plugin.STRING_SET

type job =
  | WP_None
  | WP_All
  | WP_Select of string list

val job : unit -> job
val get_behaviors : unit -> string list option
val get_properties : unit -> string list option

(** {2 Model Selection} *)

type model_kind =
  | M_Hoare
  | M_CaveatU
  | M_CaveatG
  | M_Store
  | M_Runtime

val get_model : unit -> model_kind
val get_models : unit -> string list
val get_assigns_method : unit -> Mcfg.assigns_method

module Model : Plugin.STRING
module LogicVar : Plugin.BOOL
module Assigns : Plugin.STRING

(** {2 Computation Strategies} *)

type norm = Let | Exp | Eqs | Cc
val get_norm : unit -> norm

module RTE: Plugin.BOOL
module Simpl: Plugin.BOOL
module Split: Plugin.BOOL
module Norm: Plugin.STRING
module Huge: Plugin.INT
module Axioms : Plugin.BOOL

(** {2 Prover Interface} *)

module Prover: Plugin.STRING
module Check : Plugin.STRING
module Arrays : Plugin.BOOL
module Script : Plugin.STRING
module Timeout: Plugin.INT
module Procs: Plugin.INT
module Trace: Plugin.BOOL

(** {2 Proof Obligations} *)

module Dot: Plugin.BOOL
module Print: Plugin.BOOL

(** {2 Experimental} *)

(*module Froms: Plugin.BOOL*)

(** {2 Environment Variables} *)

val get_env : ?default:string -> string -> string
val get_output : unit -> string
val get_share : unit -> string 
