(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2010                                               *)
(*    CEA   (Commissariat  l'nergie atomique et aux nergies            *)
(*           alternatives)                                                *)
(*    INRIA (Institut National de Recherche en Informatique et en         *)
(*           Automatique)                                                 *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version v2.1                *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

(* ------------------------------------------------------------------------ *)
(**{2 Updaters} *)
(* ------------------------------------------------------------------------ *)

module type Actor =
sig
  val name : string
  val kind : State.kind
  val dependencies : State.t list
end

module Updater(A : Actor) :
sig

  val set_checked : ?descr:string -> 
    Property.t -> Property.t list -> Cil_types.validity -> unit

  val set_unknown : 
    Property.t -> Property.t list -> unit

end

(* ------------------------------------------------------------------------ *)
(**{2 Indexed States} *)
(* ------------------------------------------------------------------------ *)

module type Info =
sig
  type key
  val size : int
  val kind : State.kind
  val dependencies: State.t list
  val state_name : key -> string
  val state_proxy : key -> Property.t -> (string * Property.t list) option
end

module type Indexed =
sig
  type key
  val state : key -> State.t (** Memoized state for key *)
  val property : key -> Property.t (** IPBlob of state for key *)
end

module Indexed
  (Key:Datatype.S_with_collections)
  (Info:Info with type key = Key.t) : 
  Indexed with type key = Key.t

module type Indexed2 =
sig
  type key1
  type key2
  val state : key1 -> key2 -> State.t
  val property : key1 -> key2 -> Property.t
end

module Indexed2
  (Key1:Datatype.S_with_collections)
  (Key2:Datatype.S_with_collections)
  (Info:Info with type key = Key1.t * Key2.t) : 
  Indexed2 with type key1 = Key1.t and type key2 = Key2.t


